package fr.mrdev023.vulkan_java.vk.compatibilities.validators;

import java.util.ArrayList;
import java.util.Optional;
import java.util.Set;

import org.lwjgl.vulkan.VK10;

import fr.mrdev023.vulkan_java.vk.PhysicalDevice;
import fr.mrdev023.vulkan_java.vk.Surface;
import fr.mrdev023.vulkan_java.vk.compatibilities.requirements.ComputeQueueRequirement;
import fr.mrdev023.vulkan_java.vk.compatibilities.requirements.GraphicsQueueRequirement;
import fr.mrdev023.vulkan_java.vk.compatibilities.requirements.IQueueFamilyRequirement;
import fr.mrdev023.vulkan_java.vk.compatibilities.requirements.SurfacePresentQueueRequirement;
import fr.mrdev023.vulkan_java.vk.compatibilities.requirements.TransferQueueRequirement;

public class PhysicalDeviceCompatibilityValidator {
    private int requiredQueueFlags;
    private Set<String> requiredExtensions;
    private final Optional<Surface> surface;

    public PhysicalDeviceCompatibilityValidator(int requiredQueueFlags, Set<String> requiredExtensions, Optional<Surface> surface) {
        this.requiredQueueFlags = requiredQueueFlags;
        this.requiredExtensions = requiredExtensions;
        this.surface = surface;
    }

    public Result validate(PhysicalDevice physicalDevice) {
        var queueFamilyRequirementsResult = validateQueueFamilyRequirements(physicalDevice);
        var deviceExtensionsResult = validateDeviceExtensions(physicalDevice);

        return new Result(queueFamilyRequirementsResult, deviceExtensionsResult);
    }

    private QueueFamilyRequirementsValidator.Result validateQueueFamilyRequirements(PhysicalDevice physicalDevice) {
        var queueFamilyRequirements = new ArrayList<IQueueFamilyRequirement>();

        if ((requiredQueueFlags & VK10.VK_QUEUE_GRAPHICS_BIT) != 0) {
            queueFamilyRequirements.add(new GraphicsQueueRequirement());
        }

        if ((requiredQueueFlags & VK10.VK_QUEUE_COMPUTE_BIT) != 0) {
            queueFamilyRequirements.add(new ComputeQueueRequirement());
        }

        if ((requiredQueueFlags & VK10.VK_QUEUE_TRANSFER_BIT) != 0) {
            queueFamilyRequirements.add(new TransferQueueRequirement());
        }

        if (surface.isPresent()) {
            queueFamilyRequirements.add(new SurfacePresentQueueRequirement(surface.get()));
        }

        var queueFamilyRequirementsValidator = new QueueFamilyRequirementsValidator(queueFamilyRequirements);
        var result = queueFamilyRequirementsValidator.validate(physicalDevice);

        return result;
    }

    private DeviceExtensionsValidator.Result validateDeviceExtensions(PhysicalDevice physicalDevice) {
        var deviceExtensionsValidator = new DeviceExtensionsValidator(requiredExtensions);
        var result = deviceExtensionsValidator.validate(physicalDevice);
        return result;
    }

    public static class Result {
        public final boolean isSuitable;
        public final QueueFamilyRequirementsValidator.Result queueFamilyRequirementsResult;
        public final DeviceExtensionsValidator.Result deviceExtensionsResult;

        public Result(QueueFamilyRequirementsValidator.Result queueFamilyRequirementsResult, DeviceExtensionsValidator.Result deviceExtensionsResult) {
            this.isSuitable = queueFamilyRequirementsResult.isSuitable && deviceExtensionsResult.isSuitable;
            this.queueFamilyRequirementsResult = queueFamilyRequirementsResult;
            this.deviceExtensionsResult = deviceExtensionsResult;
        }
    }
}
