package fr.mrdev023.vulkan_java.vk.compatibilities.validators;

import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import fr.mrdev023.vulkan_java.vk.PhysicalDevice;
import fr.mrdev023.vulkan_java.vk.compatibilities.requirements.IQueueFamilyRequirement;

public class QueueFamilyRequirementsValidator {
    private List<IQueueFamilyRequirement> requirements;

    public QueueFamilyRequirementsValidator(List<IQueueFamilyRequirement> requirements) {
        this.requirements = requirements;
    }
    
    /**
     * Validate the queue family requirements of the physical device.
     * 
     * <i>Note: This method return a Result to adapt the requirements to the physical device properties after the validation.</i>
     * 
     * @param physicalDevice The physical device to validate the requirements for.
     * @return A result object containing the validation results and whether the physical device is suitable.
     */
    public Result validate(PhysicalDevice physicalDevice) {
        var vkQueueFamilyProps = physicalDevice.getVkQueueFamilyProps();

        HashMap<IQueueFamilyRequirement, Optional<Integer>> result = requirements.stream()
                .collect(Collectors.toMap(
                    requirement -> requirement,
                    requirement -> Optional.empty(),
                    (a, b) -> a,
                    HashMap::new
                ));

        for (int i = 0; i < vkQueueFamilyProps.capacity(); i++) {
            var vkQueueFamilyProp = vkQueueFamilyProps.get(i);

            for (var requirement : requirements) {
                if (result.get(requirement).isPresent()) {
                    continue;
                }
                

                if (requirement.isFamilySuitable(physicalDevice, vkQueueFamilyProp, i)) {
                    result.put(requirement, Optional.of(i));
                }
            }
        }

        boolean isSuitable = result.values().stream().allMatch(Optional::isPresent);

        return new Result(result, isSuitable);
    }

    public static class Result {
        public final HashMap<IQueueFamilyRequirement, Optional<Integer>> queueFamilyIndices;
        public final boolean isSuitable;

        public Result(HashMap<IQueueFamilyRequirement, Optional<Integer>> queueFamilyIndices, boolean isSuitable) {
            this.queueFamilyIndices = queueFamilyIndices;
            this.isSuitable = isSuitable;
        }
    }
}
